/*
 * Enhanced Easy Network
 * Copyright (c) <2009>, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 3, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef __ACCEPTHANDLER_H__
#define __ACCEPTHANDLER_H__

#include <ace/Event_Handler.h>
#include <ace/Reactor.h>
#include <ace/SOCK_Acceptor.h>
#include <ace/SSL/SSL_SOCK_Acceptor.h>
/**
 * This accept handler is based on the provided solution from the ACE course.
 */
class AcceptHandler : public ACE_Event_Handler {

    private:

        /**
         * The reactor to which the accept handler belongs.
         */
        ACE_Reactor *mReactor;

        /**
         * The socket used for incoming conections.
         */
        ACE_SOCK_Acceptor mAcceptor;
		//ACE_SSL_SOCK_ACCEPTOR mAcceptor;


    public:

        /**
         * @param reactor The reactor which will use this accept handler.
         */
        AcceptHandler(ACE_Reactor *reactor = 0);

        /**
         * The destructor exists for tracing purposes.
         */
        virtual ~AcceptHandler();

        /**
         * Open the listening socket and register the handler with the reactor.
         *
         * @return 0 on success, -1 on failure
         */
        int open(void);

        /**
         * @name Overridden methods from the ACE_Event_Handler
         */
        // @{

        /**
         * Provides the handle of mAcceptor.
         */
        virtual ACE_HANDLE get_handle(void) const;

        /**
         * Create a read handler for the new connection and register that
         * handler with the reactor.
         */
        virtual int handle_input(ACE_HANDLE = ACE_INVALID_HANDLE);

        /**
         * Close the listening socket.
         */
        virtual int handle_close(ACE_HANDLE, ACE_Reactor_Mask);
        // @}
};

#endif /* __ACCEPTHANDLER_H__ */

