/*
 * Enhanced Easy Network
 * Copyright (c) <2009>, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 3, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "ENMApp.h"
#include "ScanNetworkDlg.h"

#include <wx/bitmap.h>
#include <wx/settings.h>
#include <wx/image.h>
#include <wx/string.h>

#include "ENMUtil.h"
#include "KeyDialog.h"
#include <wx/msgdlg.h>
#include <wx/dcclient.h>
#include <wx/tooltip.h>
#include <wx/thread.h>

#include "DataCenter.h"
#include "pic/icon_scan.xpm"

#if defined(__UNIX__)
#include "WirelessScanner.h"
#endif

#include "ProfileManagementControl.h"
#include "PolicyManager.h"
#include "EditProfileDialog.h"
#include "ProfileManagerDlg.h"
#include "SsidInputDialog.h"
#include "MeshInfoManager.h"
static wxMutex* s_mutexProtectingTheGlobalData = NULL;

static wxString sndLogFile(_T("/tmp/ScanNetworkDlg.log"));
static wxString srLogFile(_T("/tmp/ScanRecorder.log"));


ScanRecorder::ScanRecorder()
{
    s_mutexProtectingTheGlobalData = new wxMutex();

    for (int i = 0 ; i < MAX_AP_BUTTON_COUNT ; i ++)
    {
        m_aplist[i] = NULL;
    }
}

ScanRecorder::~ScanRecorder()
{
    Clear();
}

void ScanRecorder::Clear()
{
    ENMUtil::Log(srLogFile, _T("Enter ScanRecorder::Clear()"));
    s_mutexProtectingTheGlobalData->Lock();
    ENMUtil::Log(srLogFile, _T("\t Locked"));
    for ( int i = 0; i < MAX_AP_BUTTON_COUNT; i++ )
    {
        if ( m_aplist[i] == NULL )
            continue;
        ENMUtil::Log(srLogFile, wxString::Format(_T("\t Clear No.%d DetectedIcon: %08X (ap=%08X, button=%08X, label=%08X)."), i, m_aplist[i], m_aplist[i]->ap, m_aplist[i]->button, m_aplist[i]->txtLabel));
        ENMUtil::Log(srLogFile, _T("\t Clear content of icon."));
        if ( m_aplist[i]->button != NULL )
        {
            ENMUtil::Log(srLogFile, wxString::Format(_T("\t\t Clear button: %08X"), m_aplist[i]->button));
            m_aplist[i]->button->Show(false);
            m_aplist[i]->button->GetParent()->RemoveChild(m_aplist[i]->button);
            m_aplist[i]->button->SetParent(NULL);
            m_aplist[i]->button->Destroy();
//            delete m_aplist[i]->button;
            m_aplist[i]->button = NULL;
        }
        if ( m_aplist[i]->txtLabel != NULL )
        {
            ENMUtil::Log(srLogFile, wxString::Format(_T("\t\t Clear static label: %08X"), m_aplist[i]->txtLabel));
            m_aplist[i]->txtLabel->Show(false);
            m_aplist[i]->txtLabel->GetParent()->RemoveChild(m_aplist[i]->txtLabel);
            m_aplist[i]->txtLabel->SetParent(NULL);
            m_aplist[i]->txtLabel->Destroy();
//            delete m_aplist[i]->txtLabel;
            m_aplist[i]->txtLabel = NULL;
        }
        ENMUtil::Log(srLogFile, _T("\t Clear content of icon. OK\n"));
        ENMUtil::Log(srLogFile, _T("\t delete icon. OK"));
        delete m_aplist[i];
        m_aplist[i] = NULL;
        ENMUtil::Log(srLogFile, _T("\t Clear DetectedIcon. OK"));
    }
    s_mutexProtectingTheGlobalData->Unlock();
    ENMUtil::Log(srLogFile, _T("\t Unlocked"));
    ENMUtil::Log(srLogFile, _T("Return ScanRecorder::Clear\n"));
}

int ScanRecorder::AppendAPStatus(const APStatus& pap)
{
    ENMUtil::Log(srLogFile, wxString::Format(_T("Enter ScanRecorder::AppendAPStatus(pap=%08X)"), pap));
    s_mutexProtectingTheGlobalData->Lock();
    ENMUtil::Log(srLogFile, _T("\t Locked"));
    int position = -1;
    for (int i = 0 ; i < MAX_AP_BUTTON_COUNT ; i ++)
    {
        if (m_aplist[i] == NULL)
        {
            ENMUtil::Log(srLogFile, wxString::Format(_T("\t Find position: %d"), i));
            m_aplist[i] = new DetectedIcon();
            ENMUtil::Log(srLogFile, wxString::Format(_T("\t New DetectedIcon: %08X"), m_aplist[i]));
            m_aplist[i]->button = NULL;
            ENMUtil::Log(srLogFile, _T("\t Set button NULL"));
            m_aplist[i]->txtLabel = NULL;
            ENMUtil::Log(srLogFile, _T("\t Set static text NULL"));
            m_aplist[i]->label = _T("");
            ENMUtil::Log(srLogFile, _T("\t Set label \"\""));
            ENMUtil::Log(srLogFile, _T("\t Copy data from argument to APStatus"));
            m_aplist[i]->ap = pap;
            ENMUtil::Log(srLogFile, _T("\t Copy data OK"));
            position = i;
            break;
        }
    }
    s_mutexProtectingTheGlobalData->Unlock();
    ENMUtil::Log(srLogFile, _T("\t Unlocked"));
    ENMUtil::Log(srLogFile, wxString::Format(_T("Return ScanRecorder::AppendAPStatus : %d\n"), position));
    return position;
}

APStatus ScanRecorder::GetAPStatus(int index)
{
    APStatus result;
    memset(&result, 0, sizeof(APStatus));
    if ( m_aplist[index] != NULL )
    {
        result = m_aplist[index]->ap;
    }
    return result;
}

const long ScanNetworkDlg::ID_STATICBITMAP_BACKGROUND_IMAGE = wxNewId();
const long ScanNetworkDlg::ID_BUTTON_REFRESH = wxNewId();
const long ScanNetworkDlg::ID_BUTTON_CONNECT = wxNewId();
const long ScanNetworkDlg::ID_BUTTON_CANCEL = wxNewId();
const long ScanNetworkDlg::ID_BUTTON_DETAIL = wxNewId();
const long ScanNetworkDlg::ID_TIMER_CONNECT = wxNewId();
const long ScanNetworkDlg::ID_TIMER_ALLOCATOR = wxNewId();
const long ScanNetworkDlg::ID_TIMER_SCAN = wxNewId();
const long ScanNetworkDlg::ID_LISTCTRL_DETAIL = wxNewId();
const long ScanNetworkDlg::ID_SCROLLEDWINDOW = wxNewId();
const long ScanNetworkDlg::ID_HELP = wxNewId();


BEGIN_EVENT_TABLE(ScanNetworkDlg,wxDialog)
    EVT_INIT_DIALOG(ScanNetworkDlg::OnInit)
	EVT_CLOSE(ScanNetworkDlg::OnClose)
	EVT_WLAN_ACM(ID_WLAN_ACM_CONNECT, ScanNetworkDlg::OnWlanConnect)
	EVT_LIST_COL_BEGIN_DRAG(ID_LISTCTRL_DETAIL, ScanNetworkDlg::OnListColumnBeginDrag)
	EVT_LIST_ITEM_SELECTED(ID_LISTCTRL_DETAIL, ScanNetworkDlg::OnListCtrlDetailItemSelect)
	EVT_LIST_ITEM_DESELECTED(ID_LISTCTRL_DETAIL, ScanNetworkDlg::OnListCtrlDetailItemDeselect)
	EVT_LIST_ITEM_ACTIVATED(ID_LISTCTRL_DETAIL, ScanNetworkDlg::OnListItemActivated)
	EVT_LIST_COL_CLICK(ID_LISTCTRL_DETAIL, ScanNetworkDlg::OnListColumnClicked)
END_EVENT_TABLE()

ScanNetworkDlg::ScanNetworkDlg(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
        : wxDialog(), m_sorter(7), m_imageList(24, 24)
{
    LoadImages();
    Create(parent, wxID_ANY, _("Scan for Networks"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCLOSE_BOX);
    int width = 500, height = 480;
    int margin = 10;
    SetClientSize(wxSize(width, height));

    wxSize btnSize(100, 30);
    int x = margin;
    int y = margin;
    StaticText1 = new wxStaticText(this, wxID_ANY, _("Select a network to connect to..."), wxPoint(x, y), wxSize(325, 20));
    x = width - 120 - margin;
    y = 5;
    m_btnDetail = new wxButton(this, ID_BUTTON_DETAIL, _("&Show Detail"), wxPoint(x, y), wxSize(120, 30));

    m_bgPanel = new wxPanel(this, ID_SCROLLEDWINDOW, wxPoint(0,40), wxSize(500,400), wxBORDER_STATIC);
	m_stcBackgroundImage = new wxStaticBitmap(m_bgPanel, ID_STATICBITMAP_BACKGROUND_IMAGE, m_bmpBackground, wxPoint(0, 0), wxSize(500,400), wxSTATIC_BORDER);

	x = margin;
	y = height - btnSize.GetHeight() - 5;
    m_btnRefresh = new wxButton(this, ID_BUTTON_REFRESH, _("&Refresh"), wxPoint(x, y), btnSize);
    x = width - (btnSize.GetWidth() + margin) * 2;
    m_btnConnect = new wxButton(this, ID_BUTTON_CONNECT, _("C&onnect"), wxPoint(x, y), btnSize);
    x += btnSize.GetWidth() + margin;
    m_btnCancel = new wxButton(this, ID_BUTTON_CANCEL, _("&Cancel"), wxPoint(x, y), btnSize);

    m_listDetail = new wxListCtrl(this, ID_LISTCTRL_DETAIL, wxPoint(0,40), wxSize(width, 400), wxLC_REPORT | wxLC_SINGLE_SEL, wxDefaultValidator, _T("ID_LISTCTRL_DETAIL"));
    m_listDetail->SetImageList(&m_imageList, wxIMAGE_LIST_SMALL);
    m_listDetail->InsertColumn(0, wxEmptyString, wxLIST_FORMAT_LEFT, 0);
    m_listDetail->InsertColumn(1, _("Name"), wxLIST_FORMAT_LEFT, 120);
    m_listDetail->InsertColumn(2, _("Signal"), wxLIST_FORMAT_LEFT, 57);
    m_listDetail->InsertColumn(3, _("Encrypt"), wxLIST_FORMAT_LEFT, 85);
    m_listDetail->InsertColumn(4, _("Connected"), wxLIST_FORMAT_LEFT, 80);
    m_listDetail->InsertColumn(5, _("Type"), wxLIST_FORMAT_LEFT, 65);
    m_listDetail->InsertColumn(6, _("Channel"), wxLIST_FORMAT_LEFT, 70);
    m_listDetail->InsertColumn(7, _("MAC Address"), wxLIST_FORMAT_LEFT, 130);
    m_listDetail->Show(false);

    Connect(ID_BUTTON_REFRESH,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ScanNetworkDlg::OnButtonRefreshClick);
    Connect(ID_BUTTON_CONNECT,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ScanNetworkDlg::OnButtonConnectClick);
    Connect(ID_BUTTON_CANCEL,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ScanNetworkDlg::OnButtonCancelClick);
    Connect(ID_BUTTON_DETAIL,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ScanNetworkDlg::OnButtonDetailClick);
	m_stcBackgroundImage->Connect(ID_STATICBITMAP_BACKGROUND_IMAGE, wxEVT_LEFT_UP, (wxObjectEventFunction)&ScanNetworkDlg::OnBackgroundBitmapClick);
    Connect(ID_HELP,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ScanNetworkDlg::OnF1Press);
    Center();

    m_isForConnect = true;
    m_isDetail = false;
    m_isConnecting = false;
    m_btnConnect->Enable(false);
    m_iSelected = -1;
    m_apArray.nCount = 0;
    m_pGaugeDialog = new GaugeDialog(this);

//    wxIcon icon_scan(icon_scan_xpm);
//    SetIcon(icon_scan);
    wxIcon icon;
    wxBitmap bitmap(wxImage(::wxGetApp().strAppPath + _T("pic/Scannw.png")));
    icon.CopyFromBitmap(bitmap);
    SetIcon(icon);

    wxAcceleratorEntry entries[1];
    entries[0].Set(wxACCEL_NORMAL, WXK_F1, ID_HELP);
    wxAcceleratorTable accel(1, entries);
    SetAcceleratorTable(accel);
}

ScanNetworkDlg::~ScanNetworkDlg()
{
    m_scanRecorder.Clear();
    m_apArray.nCount = 0;
    if ( m_pGaugeDialog != NULL )
    {
        m_pGaugeDialog->Destroy();
        m_pGaugeDialog = NULL;
    }
}

unsigned int ScanNetworkDlg::GetImageIndex(const APStatus& ap, bool selected) const
{
    unsigned int uIndex = 0;
    APStatus* pCurrent = DataCenter::GetInstance()->GetCurrentAPStatus();
    if ( ap.auth != WLAN_AUTH_OPEN_NONE  )
    {
        uIndex |= 0x04;
    }
    if ( pCurrent != NULL && CompareMacAddress((unsigned char*)ap.macAddress, pCurrent->macAddress) == 0 )
    {
        uIndex |= 0x02;
    }
    if ( selected )
    {
        uIndex |= 0x01;
    }
    return uIndex;
}

void ScanNetworkDlg::LoadImages()
{
    m_bmpAp[0].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_4.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[1].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_2.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[2].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_5.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[3].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_2.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[4].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_3.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[5].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_1.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[6].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_6.png"), wxBITMAP_TYPE_PNG);
    m_bmpAp[7].LoadFile(::wxGetApp().strAppPath + _T("pic/ap_1.png"), wxBITMAP_TYPE_PNG);

    m_bmpBackground.LoadFile(::wxGetApp().strAppPath + _T("pic/network.png"), wxBITMAP_TYPE_PNG);
    m_imageList.Add(wxBitmap(wxImage(::wxGetApp().strAppPath + _T("pic/empty.png"))));

}

void ScanNetworkDlg::SetDialogMode(bool forConnect)
{
    m_isForConnect = forConnect;
}

bool ScanNetworkDlg::CloseDialog(int ret)
{
    m_pGaugeDialog->Show(false);
    m_isConnecting = false;
    if ( !IsShown() )
        return true;
    if ( IsModal() )
    {
        wxDialog::EndModal(ret);
        return true;
    }
    else
    {
        return wxDialog::Show(false);
    }
}

bool ScanNetworkDlg::GetSelectedNetwork(APStatus* pAp)
{
    if ( m_isForConnect )
        return false;
    if ( pAp == NULL )
        return false;
    memcpy(pAp, &m_toSelect, sizeof(APStatus));
    return true;
}

void ScanNetworkDlg::StartRefresh()
{
    m_btnRefresh->Enable(false);
    m_btnCancel->Enable(false);
    m_btnConnect->Enable(false);
    ClearDetectedAP();
    m_listDetail->DeleteAllItems();
    m_sorter.Reset();
#if defined(__UNIX__)
    m_pGaugeDialog->SetProcessLabel(_("Scanning for networks..."));
    m_pGaugeDialog->Show(true);
    m_isConnecting = true;
    DataCenter::GetInstance()->BeginSearchAP();
#endif
}

void ScanNetworkDlg::OnInit(wxInitDialogEvent& event)
{
    if ( m_isForConnect )
    {
        m_btnConnect->SetLabel(_("C&onnect"));
    }
    else
    {
        m_btnConnect->SetLabel(_("&OK"));
    }
    m_iSelected = -1;
    memset(&m_toSelect, 0, sizeof(APStatus));
    ShowDetail(m_isDetail = false);
    StartRefresh();
}

void ScanNetworkDlg::OnButtonCancelClick(wxCommandEvent& event)
{
    ClearDetectedAP();
    CloseDialog(wxID_CANCEL);
}

void ScanNetworkDlg::OnButtonRefreshClick(wxCommandEvent& event)
{
    StartRefresh();
}

void ScanNetworkDlg::OnButtonConnectClick(wxCommandEvent& event)
{
    if ( m_btnConnect->GetLabel().Cmp(_("&Disconnect")) == 0 )
    {
        if ( ::wxGetApp().m_nMeshStatus == MESHSTATUS_MESHSERVER )
        {
            wxMessageDialog dlg(NULL, _("Are you sure you want to disconnect current network and turn off mesh server?"),
                wxEmptyString, wxYES_NO | wxNO_DEFAULT |wxICON_EXCLAMATION);
            if(wxID_YES != dlg.ShowModal())
            {
                return;
            }
            MeshInfoManager::GetInstance()->TurnOffMesh();
        }
        m_btnConnect->Enable(false);
        DataCenter::GetInstance()->CloseWlanConnection();

    }
    else
    {
        if ( ::wxGetApp().m_nMeshStatus == MESHSTATUS_MESHSERVER )
        {
            if (::wxGetApp().m_MeshpopBalloon)
            {
                wxMessageDialog dlg(NULL, _("Are you sure you want to disconnect current network and turn off mesh server?"),
                    wxEmptyString, wxYES_NO | wxNO_DEFAULT |wxICON_EXCLAMATION);
                if(wxID_YES != dlg.ShowModal())
                {
                    return;
                }
                MeshInfoManager::GetInstance()->TurnOffMesh();
            }
        }
        ConnectAP();
    }
}

void ScanNetworkDlg::CreateNewWirelessProfile(const APStatus& ap, int priority, const wxString& name)
{
    ProfileList_t profile;
    if ( ENMUtil::IsAdministrator() )
        profile.sPermission = _T("admin");
    else
        profile.sPermission = _T("user");
	profile.sPriority = wxString::Format(_T("%d"), priority);
//    sprintf(ps, "%d", priority);
    profile.sProfileName = name;
//    profile.sProfileName = ENMUtil::WxStringToString(name);
    profile.sProfileIcon = _T("0");
    profile.sShow = _T("1");
    profile.stGenernalSet.ConnecType = _T("wireless");
	profile.stWirelessSet.SSID = wxString::FromUTF8((char*)ap.ssid.ucSsid, ap.ssid.length);
    switch( ap.type )
    {
    case WLAN_802_11_B:
        profile.stWirelessSet.WirelessType  = _T("802.11b");
        break;
    case WLAN_802_11_G:
        profile.stWirelessSet.WirelessType  = _T("802.11g");
        break;
    case WLAN_802_11_N:
        profile.stWirelessSet.WirelessType  = _T("802.11n");
        break;
    default:
        profile.stWirelessSet.WirelessType  = _T("Automatic");
        break;
    }
	profile.stWirelessSet.stWEPSecurity.keyIndex = wxString::Format(_T("%d"), m_iKeyIdx);
    switch( ap.auth )
    {
    case WLAN_AUTH_OPEN_NONE :
        profile.stWirelessSet.WlanSecurity = _T("open-none");
        break;
    case WLAN_AUTH_OPEN_WEP:
        profile.stWirelessSet.WlanSecurity = _T("open-wep");
        profile.stWirelessSet.stWEPSecurity.value = m_strKey;
        break;
    case WLAN_AUTH_WPAPSK_AES:
        profile.stWirelessSet.WlanSecurity = _T("wpa-psk-aes");
        profile.stWirelessSet.stWPASecurity.value = m_strKey;
        break;
    case WLAN_AUTH_WPAPSK_TKIP:
        profile.stWirelessSet.WlanSecurity = _T("wpa-psk-tkip");
        profile.stWirelessSet.stWPASecurity.value = m_strKey;
        break;
    case WLAN_AUTH_WPA2PSK_AES:
        profile.stWirelessSet.WlanSecurity = _T("wpa2-psk-aes");
        profile.stWirelessSet.stWPASecurity.value = m_strKey;
        break;
    case WLAN_AUTH_WPA2PSK_TKIP:
        profile.stWirelessSet.WlanSecurity = _T("wpa2-psk-tkip");
        profile.stWirelessSet.stWPASecurity.value = m_strKey;
        break;
    default:
        profile.stWirelessSet.WlanSecurity = _T("unknown");
        break;
    }
    profile.stOtherSet.NetworkSecurity = _T("0");
    profile.stOtherSet.AutoRunAppFlag = _T("0");
    profile.stOtherSet.IPDNSFlag = _T("0");
    profile.stOtherSet.IsHomepage = _T("0");
    profile.stOtherSet.ProxyFlag = _T("0");
    EditProfileDialog editDialog(this, profile, true);
    if ( editDialog.ShowModal() == wxID_OK )
    {
        profile = editDialog.GetProfile();
        ProfileManagementControl* pProfileController = DataCenter::GetInstance()->GetProfileController();
        if ( pProfileController->InsertNewWirelessProfile(profile) )
        {
            ProfileManagerDlg* pProfileManagerDlg = DataCenter::GetInstance()->GetProfileManagerDlgHandle();
            pProfileManagerDlg->InsertNewProfile(profile);
            wxMessageDialog dialog(this, _("Profile created successfully."), wxEmptyString, wxOK | wxICON_INFORMATION);
            dialog.ShowModal();
        }
        else
        {
            wxMessageDialog dialog(this, _("Failed to create profile."), wxEmptyString, wxOK | wxICON_EXCLAMATION);
            dialog.ShowModal();
        }
    }
}

void ScanNetworkDlg::OnWlanConnect(WlanAcmEvent& event)
{
    if ( !IsShown() )
        return;

    ENMUtil::Log(sndLogFile, _T("Enter ScanNetworkDlg::OnWlanConnect"));
    unsigned long code = event.GetEventCode();
    if ( code == WLAN_ACM_EVENT_CONNECT_SUCCESS )
    {
        ENMUtil::Log(sndLogFile, _T("\t connect success"));
        m_pGaugeDialog->Show(false);
        m_isConnecting = false;
        wxString ssid = wxString::FromUTF8((char*)m_toConnect.ssid.ucSsid, m_toConnect.ssid.length);
        if ( ssid.Cmp(wxT("Guest")) == 0 )//Do not create Guest.profile
        {
            return;
        }

        wxString message = _("Successfully connected to ") + ssid + _T(".");
        ProfileManagementControl* pProfileController = DataCenter::GetInstance()->GetProfileController();
        int checkResult = pProfileController->CheckWirelessProfileExist(m_toConnect);
        if ( checkResult == 0 )
            return;
        PolicyManager* pPolicyManager = DataCenter::GetInstance()->GetPolicyManager();
        if ( (checkResult & 0x00F00000) == 0 && pPolicyManager->IsAllowOwnerProfile() )
        {
            if ( (checkResult & 0x000F0000) == 0x00010000 )
            {
                ssid += _T("_0");
            }
            message += _("\nWould you like to create a new profile for this network?");
            wxMessageDialog dialog(this, message, wxEmptyString, wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION);
            if ( dialog.ShowModal() == wxID_YES )
            {
                int priority = checkResult & 0x0000FFFF;
                CreateNewWirelessProfile(m_toConnect, priority + 1, ssid);
            }
        }
    }
    if ( code == WLAN_ACM_EVENT_CONNECT_FAILED )
    {
        ENMUtil::Log(sndLogFile, _T("\t connect failed"));
        m_pGaugeDialog->Show(false);
        m_isConnecting = false;
        unsigned long error = event.GetErrorCode();
        wxString message = _("Cannot connect. \n\nPlease check the network authentication type and \nmake sure the password is correct.");
        switch ( error )
        {
        case WLAN_ACM_ERROR_NO_INTERFACE:
            message = _("Cannot find wireless adapter!");
            break;
        case WLAN_ACM_ERROR_NO_RESPONSE:
            message = _("No response received from selected network.");
            break;
        };
        wxMessageDialog msg(this, message, wxEmptyString, wxOK | wxICON_EXCLAMATION);
        msg.ShowModal();
    }
    if ( code == WLAN_ACM_EVENT_DISCONNECTED && m_isForConnect )
    {
        ENMUtil::Log(sndLogFile, _T("\t disconnect"));
//        m_btnConnect->Enable(true);
        m_btnConnect->SetLabel(_("C&onnect"));
    }
//    RefreshIcons(m_iSelected);
    if ( !m_pGaugeDialog->IsShown() )
    {
        m_listDetail->DeleteAllItems();
        ENMUtil::Log(sndLogFile, _T("\t ClearDetectedAP"));
        ClearDetectedAP();
        ENMUtil::Log(sndLogFile, _T("\t UpdateUI"));
        UpdateUI(m_apArray);
        m_btnConnect->Enable(m_iSelected != -1);
        m_btnRefresh->Enable(true);
        m_btnCancel->Enable(true);
    }
    ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::OnWlanConnect\n"));
}

void ScanNetworkDlg::OnBackgroundBitmapClick(wxMouseEvent& event)
{
    DataCenter::GetInstance()->GetScanNetworkDlgHandle()->ProcessBackgroundClick();
}

void ScanNetworkDlg::OnBitmapButtonClick(wxCommandEvent& event)
{
    int id = event.GetId();
    int index = id - MAX_AP_BUTTON_ID;
    if (index >= 0 && index < MAX_AP_BUTTON_ID)
    {
        m_iSelected = index;
        if ( m_iSelected < m_listDetail->GetItemCount() )
        {
            m_listDetail->SetItemState(m_iSelected, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        }
        else
        {
            DeselectAllListItem();
        }
//#if defined(__UNIX__)
//        m_btnConnect->Enable(true);
//#endif
        RefreshIcons(m_iSelected);
    }
}

void ScanNetworkDlg::OnBitmapButtonDblClick(wxMouseEvent& event)
{
    DataCenter::GetInstance()->GetScanNetworkDlgHandle()->ProcessDoubleClick();
}

void ScanNetworkDlg::OnButtonDetailClick(wxCommandEvent& event)
{
    ShowDetail(!m_isDetail);
    m_isDetail = !m_isDetail;
}

void ScanNetworkDlg::ProcessDoubleClick()
{
    m_btnConnect->Enable(false);
    APStatus selectedAP = m_scanRecorder.GetAPStatus(m_iSelected);
    APStatus* pCurrentAP = DataCenter::GetInstance()->GetCurrentAPStatus();
//    if ( m_isForConnect && pCurrentAP != NULL && selectedAP.ssid == pCurrentAP->ssid )
    if ( m_isForConnect && pCurrentAP != NULL && CompareMacAddress(selectedAP.macAddress, pCurrentAP->macAddress) == 0 )
    {
        if ( ::wxGetApp().m_nMeshStatus == MESHSTATUS_MESHSERVER )
        {
            wxMessageDialog dlg(NULL, _("Are you sure you want to disconnect current network and turn off mesh server?"),
                wxEmptyString, wxYES_NO | wxNO_DEFAULT |wxICON_EXCLAMATION);
            if(wxID_YES != dlg.ShowModal())
            {
                return;
            }
            MeshInfoManager::GetInstance()->TurnOffMesh();
        }
        DataCenter::GetInstance()->CloseWlanConnection();
    }
    else
    {
        if ( ::wxGetApp().m_nMeshStatus == MESHSTATUS_MESHSERVER )
        {
            if (::wxGetApp().m_MeshpopBalloon)
            {
                wxMessageDialog dlg(NULL, _("Are you sure you want to disconnect current network and turn off mesh server?"),
                    wxEmptyString, wxYES_NO | wxNO_DEFAULT |wxICON_EXCLAMATION);
                if(wxID_YES != dlg.ShowModal())
                {
                    return;
                }
                MeshInfoManager::GetInstance()->TurnOffMesh();
            }

        }
        ConnectAP();
    }
}

void ScanNetworkDlg::ProcessBackgroundClick()
{
    m_btnConnect->Enable(false);
    RefreshIcons(m_iSelected = -1);
    m_listDetail->DeleteAllItems();
    for ( int i = 0; i < m_apArray.nCount; i++ )
    {
        ShowAPInfo(m_apArray.apList[i], i);
    }
    DeselectAllListItem();
    wxListEvent event(wxEVT_COMMAND_LIST_ITEM_DESELECTED, m_listDetail->GetId());
    ProcessEvent(event);
}

void ScanNetworkDlg::ConnectAP()
{
    ENMUtil::Log(sndLogFile, _T("Enter ScanNetworkDlg::ConnectAP"));
    DataCenter* pDataCenter = DataCenter::GetInstance();
    m_btnConnect->Enable(false);
    m_btnRefresh->Enable(false);
    m_btnCancel->Enable(false);
    APStatus ap;
    if ( m_isDetail )
    {
        long nItem = m_listDetail->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if ( nItem == -1 )
        {
            m_btnRefresh->Enable(true);
            m_btnCancel->Enable(true);
            ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, no select\n"));
            return;
        }
        ap = m_scanRecorder.GetAPStatus(nItem);
    }
    else
    {
        if ( m_iSelected == -1 )
        {
            m_btnRefresh->Enable(true);
            m_btnCancel->Enable(true);
            ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, no select\n"));
            return;
        }
        ap = m_scanRecorder.GetAPStatus(m_iSelected);
    }

    if ( !m_isForConnect )
    {
        m_toSelect = ap;
        m_btnRefresh->Enable(true);
        m_btnConnect->Enable(true);
        m_btnCancel->Enable(true);
        CloseDialog(wxID_OK);
        ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, not for connect\n"));
        return;
    }

    if ( !CheckSsid(&ap) )
    {
        m_strKey.Clear();
        m_iKeyIdx = 0;
        m_btnConnect->Enable(true);
        m_btnRefresh->Enable(true);
        m_btnCancel->Enable(true);
        ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, canceled\n"));
        return;
    }

//    if ( ap.auth == WLAN_SECURITY_WEP || ap.auth == WLAN_SECURITY_WPA_PSK )
	WlanEncryptType encType = ENCRYPT(ap.auth);
	if ( encType == ENCRYPT_WEP || encType == ENCRYPT_TKIP || encType == ENCRYPT_AES )
    {
        KeyDialog dlg(this, encType == ENCRYPT_WEP);
        m_iKeyIdx = 0;
        if ( dlg.ShowModal() == wxID_OK )
        {
            m_strKey = dlg.GetKey();
            if ( encType == ENCRYPT_WEP )
            {
                m_iKeyIdx = dlg.GetIdx();
            }
        }
        else
        {
            m_strKey.Clear();
            m_iKeyIdx = 0;
            m_btnConnect->Enable(true);
            m_btnRefresh->Enable(true);
            m_btnCancel->Enable(true);
            ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, canceled\n"));
            return;
        }
    }
    else if ( ap.auth == WLAN_AUTH_UNKNOWN )
    {
        wxMessageDialog msg(this, _("Unknown authentication mode!"), wxEmptyString, wxOK | wxICON_INFORMATION);
        msg.ShowModal();
        m_btnRefresh->Enable(true);
        m_btnCancel->Enable(true);
        ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, unknown authentication mode\n"));
        return;
    }
    else
    {
        m_iKeyIdx = 0;
    }

#if defined(__UNIX__)
    wxString msg = _("Connecting to ") + wxString::FromUTF8((char*)ap.ssid.ucSsid, ap.ssid.length) + _T(" ...");
    m_pGaugeDialog->SetProcessLabel(msg);
    m_pGaugeDialog->Show(true);
    m_isConnecting = true;
    pDataCenter->CloseWlanConnection(true);
    m_toConnect = ap;
    m_btnConnect->Enable(false);
    pDataCenter->ConnectWlanNetwork(ap, m_strKey, m_iKeyIdx, false);
#endif
    ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ConnectAP, OK\n"));
}

bool ScanNetworkDlg::CheckSsid(APStatus* pAPStatus)
{
    if ( pAPStatus == NULL )
        return false;

    if ( pAPStatus->ssid.length > 0 )
        return true;

    SsidInputDialog dialog(this);
    if ( dialog.ShowModal() != wxID_OK )
        return false;

    wxString ssid = dialog.GetSsid();
    string _s = ENMUtil::WxStringToString(ssid);
    const char* pszSsid = _s.c_str();
    pAPStatus->ssid.length = ssid.Length();
    strcpy((char*)pAPStatus->ssid.ucSsid, pszSsid);
    return true;
}

void ScanNetworkDlg::ShowDetail(bool detail)
{
    m_listDetail->Show(detail);
    m_listDetail->Enable(detail);
    m_bgPanel->Show(!detail);
    m_bgPanel->Enable(!detail);
    for (size_t i = 0 ; i < MAX_AP_BUTTON_COUNT ; i ++)
    {
        if ( m_scanRecorder.m_aplist[i] != NULL
                && m_scanRecorder.m_aplist[i]->button != NULL
                && m_scanRecorder.m_aplist[i]->txtLabel != NULL )
        {
            m_scanRecorder.m_aplist[i]->button->Show(!detail);
            m_scanRecorder.m_aplist[i]->txtLabel->Show(!detail);
        }
    }
    if ( detail )
    {
        m_btnDetail->SetLabel(_("<< &Back"));
    }
    else
    {
        m_btnDetail->SetLabel(_("&Show Detail"));
        RefreshIcons(m_iSelected);
    }
}

void ScanNetworkDlg::OnClose(wxCloseEvent& event)
{
    m_pGaugeDialog->Show(false);
    m_isConnecting = false;
    if ( event.CanVeto() )
        event.Veto();
    ClearDetectedAP();
    CloseDialog(wxID_CANCEL);
}

void ScanNetworkDlg::ClearDetectedAP()
{
    ENMUtil::Log(sndLogFile, _T("Enter ScanNetworkDlg::ClearDetectedAP()"));
    ENMUtil::Log(sndLogFile, _T("\t Clear ScanRecorder."));
    m_scanRecorder.Clear();
    ENMUtil::Log(sndLogFile, _T("\t Clear ScanRecorder OK.\n"));
    ENMUtil::Log(sndLogFile, _T("\t Reset IconPositionCalculator."));
    calculator.Reset();
    ENMUtil::Log(sndLogFile, _T("\t Reset IconPositionCalculator OK.\n"));
    ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::ClearDetectedAP\n"));
}

void ScanNetworkDlg::DrawButton(const wxPoint& centerPos, int id, const wxString& label, const size_t charsOfLabel)
{
    wxPoint buttonPos(centerPos.x - AP_ICON_SIZE.GetWidth() / 2, centerPos.y - AP_ICON_SIZE.GetHeight() / 2);
#if defined(__UNIX__)
    wxPoint labelPos(buttonPos.x + AP_ICON_SIZE.GetWidth() + 12, buttonPos.y + (AP_ICON_SIZE.GetHeight() - AP_LABEL_SIZE.GetHeight()) / 2 + 3);
    wxBitmapButton* pButton = new wxBitmapButton(m_bgPanel, MAX_AP_BUTTON_ID + id, m_bmpAp[0], buttonPos, wxDefaultSize, wxBORDER_NONE);
#endif
    wxToolTip* pTip = new wxToolTip(label);
    pButton->SetToolTip(pTip);
    Connect(MAX_AP_BUTTON_ID + id, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&ScanNetworkDlg::OnBitmapButtonClick);
    pButton->Connect(MAX_AP_BUTTON_ID + id, wxEVT_LEFT_DCLICK, (wxObjectEventFunction)&ScanNetworkDlg::OnBitmapButtonDblClick);
    wxString newLabel = label.Length() > 0 ? label : wxString(_("<Hidden>"));
    if ( newLabel.Length() > charsOfLabel && newLabel.Cmp(_("<Hidden>")) != 0 )
    {
        newLabel = label.SubString(0, charsOfLabel) + _("...");
    }

#if defined(__UNIX__)
    wxStaticText* pTextLabel = new wxStaticText(m_bgPanel, wxID_ANY, newLabel, labelPos, AP_LABEL_SIZE, wxALIGN_LEFT);
#endif

    size_t iBmpIndex = GetImageIndex(m_scanRecorder.m_aplist[id]->ap, false);
    pButton->SetBitmapLabel(m_bmpAp[iBmpIndex]);
    if ( (iBmpIndex & 0x02) == 0x02 )
    {
        pTextLabel->SetForegroundColour(wxColour(255, 0, 0));
    }
    else
    {
        pTextLabel->SetForegroundColour(wxColour(0, 0, 0));
    }
    pTextLabel->Refresh();

    m_scanRecorder.m_aplist[id]->button = pButton;
    m_scanRecorder.m_aplist[id]->txtLabel = pTextLabel;
    m_scanRecorder.m_aplist[id]->label = label;

    pButton->Show(!m_isDetail);
    pTextLabel->Show(!m_isDetail);
}

void ScanNetworkDlg::RefreshIcons(int selected)
{
    ENMUtil::Log(sndLogFile, wxString::Format(_T("Enter ScanNetworkDlg::RefreshIcons(%d)"), selected));
    APStatus* pCurAP = DataCenter::GetInstance()->GetCurrentAPStatus();
    ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call DataCenter::GetCurrentAPStatus() : %08X"), pCurAP));
    ENMUtil::Log(sndLogFile, _T("\t Begin to refresh icons."));
    for (int i = 0 ; i < MAX_AP_BUTTON_COUNT ; i ++)
    {
        if ( m_scanRecorder.m_aplist[i] == NULL )
            continue;
        DetectedIcon* pIcon = m_scanRecorder.m_aplist[i];
        APStatus ap = pIcon->ap;
        wxBitmapButton* pButton = pIcon->button;
        wxStaticText* pText = pIcon->txtLabel;
        ENMUtil::Log(sndLogFile, wxString::Format(_T("\t\t Refresh No.%d icon: %08X (ap=%08X, button=%08X, stx=%08X)"), i, pIcon, &ap, pButton, pText));
        if ( pIcon != NULL && pButton != NULL && pText != NULL )
        {
            unsigned int iBmpIndex = GetImageIndex(ap, i == selected);
            pButton->SetBitmapLabel(m_bmpAp[iBmpIndex]);
            if ( (iBmpIndex & 0x02) == 0x02 )
            {
                pText->SetForegroundColour(wxColour(255, 0, 0));
            }
            else
            {
                pText->SetForegroundColour(wxColour(0, 0, 0));
            }
            pText->Refresh();
        }
        ENMUtil::Log(sndLogFile, wxString::Format(_T("\t\t Finish refresh No.%d icon."), i));
    }
    ENMUtil::Log(sndLogFile, _T("\t Finish to refresh icons."));

    if ( m_isForConnect )
    {
        ENMUtil::Log(sndLogFile, _T("\t ScanNetworkDlg is used for connect network."));
        APStatus* pSelAP = NULL;
        if ( selected >= 0 && selected < MAX_AP_BUTTON_COUNT && m_scanRecorder.m_aplist[selected] != NULL )
        {
            pSelAP = &m_scanRecorder.m_aplist[selected]->ap;
        }
        if ( pSelAP != NULL && pCurAP != NULL && CompareMacAddress(pSelAP->macAddress, pCurAP->macAddress) == 0 )
//        if ( pSelAP != NULL && pCurAP != NULL && pSelAP->ssid == pCurAP->ssid )
        {
            ENMUtil::Log(sndLogFile, _T("\t\t Set button label is \"Disconnect\"."));
            m_btnConnect->SetLabel(_("&Disconnect"));
        }
        else
        {
            ENMUtil::Log(sndLogFile, _T("\t\t Set button label is \"Connect\"."));
            m_btnConnect->SetLabel(_("C&onnect"));
        }
    }
    ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::RefreshIcons\n"));
}

void ScanNetworkDlg::UI_UpdateNetworkMode(NETWORK_MODE networkMode)
{
    switch ( networkMode )
    {
    case NETWORK_MODE_WIRED_LAN_CONNECT:
    case NETWORK_MODE_WIRED_LAN_DISCONNECT:
        m_btnRefresh->Enable(false);
        break;
    case NETWORK_MODE_WIRELESS_LAN_CONNECT:
    case NETWORK_MODE_WIRELESS_LAN_DISCONNECT:
        m_btnRefresh->Enable(true);
        break;
    case NETWORK_MODE_MESH_CONNECT:
    case NETWORK_MODE_MESH_DISCONNECT:
        break;
    case NETWORK_MODE_DISCONNECTED:
        m_btnRefresh->Enable(false);
        break;
    default:
        break;
    }
}

void ScanNetworkDlg::UI_UpdateNetworkStatusInfo(NETWORK_MODE networkMode, UI_NETWORK_STATUS_INFO& statusInfo)
{
    // Do nothing
}

void ScanNetworkDlg::UI_UpdateAPSatus(WlanResult result, const AccessPointList& apList)
{
    m_pGaugeDialog->Show(false);
    m_isConnecting = false;
    if ( !IsShown() )
        return;

    ENMUtil::Log(sndLogFile, wxString::Format(_T("Enter ScanNetworkDlg::UI_UpdateAPSatus(%d, %08X)"), result, &apList));
    ENMUtil::Log(sndLogFile, wxString::Format(_T("\t Count of detected AP: %d"), apList.nCount));
    if ( apList.nCount > 0 )
    {
        ENMUtil::Log(sndLogFile, _T("\t Clear m_apArray and copy argument data"));
        m_apArray.nCount = apList.nCount;
        memset((char*)m_apArray.apList, 0, sizeof(APStatus) * apList.nCount);
        memcpy((char*)m_apArray.apList, (char*)apList.apList, sizeof(APStatus) * apList.nCount);
        ENMUtil::Log(sndLogFile, _T("\t Copy argument OK"));
    }

    m_iSelected = -1;
    ENMUtil::Log(sndLogFile, _T("\t Set button status (true, false, true)"));
    m_btnRefresh->Enable(true);
    m_btnConnect->Enable(false);
    m_btnCancel->Enable(true);
    ENMUtil::Log(sndLogFile, _T("\t Set button status OK"));
    if ( result == WLAN_SUCCESS || result == WLAN_DATA_LOST )
    {
        ENMUtil::Log(sndLogFile, _T("\t Scan success, Begin update UI"));
        UpdateUI(apList);
        ENMUtil::Log(sndLogFile, _T("\t Update UI OK"));
    }
    else if (result == WLAN_NO_INTERFACE)
    {
        wxMessageDialog msg(this, _("Cannot find WLAN adapter!"), wxEmptyString, wxOK | wxICON_INFORMATION);
        msg.ShowModal();
    }
    else
    {
        wxMessageDialog msg(this, _("Failed to scan for networks!"), wxEmptyString, wxOK | wxICON_INFORMATION);
        msg.ShowModal();
    }
    m_btnCancel->SetFocus();
    ENMUtil::Log(sndLogFile, _T("\t Return ScanNetworkDlg::UI_UpdateAPSatus\n"));
}

void ScanNetworkDlg::UpdateUI(const AccessPointList& apList)
{
    ENMUtil::Log(sndLogFile, wxString::Format(_T("Enter ScanNetworkDlg::UpdateUI(%08X)"), &apList));
//    m_iSelected = -1;
    ENMUtil::Log(sndLogFile, _T("\t Set button status (true, false, true)"));
    m_btnRefresh->Enable(true);
    m_btnConnect->Enable(false);
    m_btnCancel->Enable(true);
    ENMUtil::Log(sndLogFile, _T("\t Set button status OK"));
    ENMUtil::Log(sndLogFile, wxString::Format(_T("\t Update detected ap, count: %d"), apList.nCount));
    for ( int i = 0 ; i < apList.nCount ; i++ )
    {
        APStatus ap = apList.apList[i];
        ENMUtil::Log(sndLogFile, _T("\t Append detected ap: ") + wxString::FromUTF8((char*)ap.ssid.ucSsid, ap.ssid.length));
        int id = m_scanRecorder.AppendAPStatus(ap);
        ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call ScanRecorder::AppendAPStatus(%08X) : %d"), &ap, id));
        wxPoint pos;
        int charsOfLabel = 0;
        ENMUtil::Log(sndLogFile, _T("\t Calculating icon position..."));
        if ( calculator.CalculatePosition(ap, pos, &charsOfLabel) && id != -1 )
        {
            ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call IconPositionCalculator::CalculatePosition(%08X, [%d, %d], [%d])"), &ap, pos.x, pos.y, charsOfLabel));
            wxString _strApName = wxString::FromUTF8((char*)ap.ssid.ucSsid, ap.ssid.length);
//            ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call ScanNetworkDlg::DrawButton([%d, %d], %d, %s, %d)"), pos.x, pos.y, id, _strApName, charsOfLabel));
            DrawButton(pos, id, _strApName, charsOfLabel);
            ENMUtil::Log(sndLogFile, _T("\t Draw button OK"));
        }
        ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call ScanRecorder::ShowAPInfo(%08X, %d)"), &ap, i));
        ShowAPInfo(ap, i);
        ENMUtil::Log(sndLogFile, _T("\t Show ap info in list control OK"));
        if ( i == m_iSelected )
        {
            m_listDetail->SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        }
    }
    ENMUtil::Log(sndLogFile, _T("\t Update detected ap OK"));
//    RefreshIcons(m_iSelected = -1);
    ENMUtil::Log(sndLogFile, wxString::Format(_T("\t call RefreshIcons(%d)"), m_iSelected));
    RefreshIcons(m_iSelected);
    ENMUtil::Log(sndLogFile, _T("\t RefreshIcons OK"));
    ENMUtil::Log(sndLogFile, _T("Return ScanNetworkDlg::UpdateUI\n"));
}

void ScanNetworkDlg::ShowAPInfo(const APStatus& apStatus, long nItem)
{
    wxString name = wxString::FromUTF8((char*)apStatus.ssid.ucSsid, apStatus.ssid.length);
    if ( name.Length() == 0 )
        name = _("<Hidden>");
    m_listDetail->InsertItem(nItem, wxEmptyString, 0);
    int nSubItem = 1;
    m_listDetail->SetItem(nItem, nSubItem++, name);

    m_listDetail->SetItem(nItem, nSubItem++, wxString::Format(_T("%d%%"), apStatus.signalQuality));

    if ( apStatus.auth == WLAN_AUTH_OPEN_NONE )
        m_listDetail->SetItem(nItem, nSubItem++, _("Unencrypted"));
    else
        m_listDetail->SetItem(nItem, nSubItem++, _("Encrypted"));

    APStatus* pCurAp = DataCenter::GetInstance()->GetCurrentAPStatus();
    if ( pCurAp != NULL && CompareMacAddress(pCurAp->macAddress, (unsigned char*)apStatus.macAddress) == 0 )
        m_listDetail->SetItem(nItem, nSubItem++, _("Yes"));
    else
        m_listDetail->SetItem(nItem, nSubItem++, _("No"));

    if ( apStatus.type == WLAN_802_11_B )
        m_listDetail->SetItem(nItem, nSubItem++, _T("802.11b"));
    else if ( apStatus.type == WLAN_802_11_G )
        m_listDetail->SetItem(nItem, nSubItem++, _T("802.11g"));
    else if ( apStatus.type == WLAN_802_11_N )
        m_listDetail->SetItem(nItem, nSubItem++, _T("802.11n"));
	else
        m_listDetail->SetItem(nItem, nSubItem++, _("Unknown"));

    m_listDetail->SetItem(nItem, nSubItem++, wxString::Format(_T("%d"), apStatus.channel));
    m_listDetail->SetItem(nItem, nSubItem++, MacAddressToString(apStatus.macAddress));

    if ( pCurAp != NULL && CompareMacAddress((unsigned char*)apStatus.macAddress, pCurAp->macAddress) == 0 )
    {
        m_listDetail->SetItemTextColour(nItem, wxColour(255, 0, 0));
    }
}

void ScanNetworkDlg::OnListColumnBeginDrag(wxListEvent& event)
{
    if ( event.GetColumn() == 0 )
    {
        event.Veto();
    }
}

void ScanNetworkDlg::OnListCtrlDetailItemSelect(wxListEvent& event)
{
    m_iSelected = m_listDetail->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
//    wxMessageBox(wxString::Format(_T("selected: %d"), m_iSelected), _T("list item"));
    wxListItem item;
    item.SetId(m_iSelected);
    item.SetColumn(7);
    item.SetMask(wxLIST_MASK_TEXT);
    m_listDetail->GetItem(item);
    wxString itemText = item.GetText();
//    wxString itemText = m_listDetail->GetItemText(m_iSelected);
    APStatus* pCurrentAP = DataCenter::GetInstance()->GetCurrentAPStatus();
    if ( m_isForConnect )
    {
        unsigned char pszMac[6] = { 0 };
        string _s = ENMUtil::WxStringToString(itemText);
        StringToMacAddress(pszMac, _s.c_str(), _s.length());
//        if ( pCurrentAP != NULL && itemText.Cmp(pCurrentAP->name) == 0 )
        if ( pCurrentAP != NULL && CompareMacAddress(pszMac, pCurrentAP->macAddress) == 0 )
        {
            m_btnConnect->SetLabel(_("&Disconnect"));
        }
        else
        {
            m_btnConnect->SetLabel(_("C&onnect"));
        }
    }
    m_btnConnect->Enable(true);
}

void ScanNetworkDlg::OnListCtrlDetailItemDeselect(wxListEvent& event)
{
    m_iSelected = -1;
    if ( m_isForConnect )
    {
        m_btnConnect->SetLabel(_("C&onnect"));
    }
    m_btnConnect->Enable(false);
}

void ScanNetworkDlg::OnListItemActivated(wxListEvent& event)
{
    ProcessDoubleClick();
}

void ScanNetworkDlg::OnListColumnClicked(wxListEvent& event)
{
    if ( event.GetColumn() == 0 )
        return;
    if ( m_sorter.Sort(m_apArray.apList, m_apArray.nCount, event.GetColumn() - 1) )
    {
        ClearDetectedAP();
        m_listDetail->DeleteAllItems();
        UpdateUI(m_apArray);
    }
}

void ScanNetworkDlg::OnF1Press(wxCommandEvent& event)
{
#if defined(__UNIX__)
    if(wxFileExists(wxT("/usr/share/doc/enhanced-easy-network/Intel_CMPC2.0_Enhanced_Easy_Network_User_Manual_Ubuntu.pdf.gz")))
    {
        wxString cmd = wxT("evince /usr/share/doc/enhanced-easy-network/Intel_CMPC2.0_Enhanced_Easy_Network_User_Manual_Ubuntu.pdf.gz --page-label=20");
        //wxExecute(cmd);
    }
#endif
}

void ScanNetworkDlg::DeselectAllListItem()
{
    int iItemCount = m_listDetail->GetItemCount();
    for ( int item = 0; item < iItemCount; item++ )
    {
        m_listDetail->SetItemState(item, ~wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    }
}

bool ScanNetworkDlg::IsConnectingAP() const
{
    return m_isConnecting;
}
